@@ -70,7 +70,25 @@ type_dict = {
     "size_t"  : { "j_type" : "long", "jn_type" : "long", "jni_type" : "jlong", "suffix" : "J" },
     "__int64" : { "j_type" : "long", "jn_type" : "long", "jni_type" : "jlong", "suffix" : "J" },
     "int64"   : { "j_type" : "long", "jn_type" : "long", "jni_type" : "jlong", "suffix" : "J" },
-    "double[]": { "j_type" : "double[]", "jn_type" : "double[]", "jni_type" : "jdoubleArray", "suffix" : "_3D" }
+    "double[]": { "j_type" : "double[]", "jn_type" : "double[]", "jni_type" : "jdoubleArray", "suffix" : "_3D" },
+    'string'  : {  # std::string, see "String" in modules/core/misc/java/gen_dict.json
+        'j_type': 'String',
+        'jn_type': 'String',
+        'jni_name': 'n_%(n)s',
+        'jni_type': 'jstring',
+        'jni_var': 'const char* utf_%(n)s = env->GetStringUTFChars(%(n)s, 0); std::string n_%(n)s( utf_%(n)s ? utf_%(n)s : "" ); env->ReleaseStringUTFChars(%(n)s, utf_%(n)s)',
+        'suffix': 'Ljava_lang_String_2',
+        'j_import': 'java.lang.String'
+    },
+    'vector_string': {  # std::vector<std::string>, see "vector_String" in modules/core/misc/java/gen_dict.json
+        'j_type': 'List<String>',
+        'jn_type': 'List<String>',
+        'jni_type': 'jobject',
+        'jni_var': 'std::vector< std::string > %(n)s',
+        'suffix': 'Ljava_util_List',
+        'v_type': 'string',
+        'j_import': 'java.lang.String'
+    },
 }
 
 # Defines a rule to add extra prefixes for names from specific namespaces.
@@ -105,7 +123,7 @@ T_CPP_MODULE = Template(read_contents(os.path.join(SCRIPT_DIR, 'templates/cpp_mo
 
 class GeneralInfo():
     def __init__(self, type, decl, namespaces):
-        self.symbol_id, self.namespace, self.classpath, self.classname, self.name = self.parseName(decl[0], namespaces)
+        self.namespace, self.classpath, self.classname, self.name = self.parseName(decl[0], namespaces)
 
         # parse doxygen comments
         self.params={}
@@ -141,13 +159,13 @@ class GeneralInfo():
                 break
         pieces = localName.split(".")
         if len(pieces) > 2: # <class>.<class>.<class>.<name>
-            return name, spaceName, ".".join(pieces[:-1]), pieces[-2], pieces[-1]
+            return spaceName, ".".join(pieces[:-1]), pieces[-2], pieces[-1]
         elif len(pieces) == 2: # <class>.<name>
-            return name, spaceName, pieces[0], pieces[0], pieces[1]
+            return spaceName, pieces[0], pieces[0], pieces[1]
         elif len(pieces) == 1: # <name>
-            return name, spaceName, "", "", pieces[0]
+            return spaceName, "", "", pieces[0]
         else:
-            return name, spaceName, "", "" # error?!
+            return spaceName, "", "" # error?!
 
     def fullName(self, isCPP=False):
         result = ".".join([self.fullClass(), self.name])
@@ -224,9 +242,6 @@ class ClassInfo(GeneralInfo):
         for m in decl[2]:
             if m.startswith("="):
                 self.jname = m[1:]
-            if m == '/Simple':
-                self.smart = False
-
         self.base = ''
         if decl[1]:
             #self.base = re.sub(r"\b"+self.jname+r"\b", "", decl[1].replace(":", "")).strip()
@@ -252,8 +267,8 @@ class ClassInfo(GeneralInfo):
 
     def getAllMethods(self):
         result = []
-        result += [fi for fi in self.methods if fi.isconstructor]
-        result += [fi for fi in self.methods if not fi.isconstructor]
+        result.extend([fi for fi in sorted(self.methods) if fi.isconstructor])
+        result.extend([fi for fi in sorted(self.methods) if not fi.isconstructor])
         return result
 
     def addMethod(self, fi):
@@ -372,8 +387,8 @@ class JavaWrapperGenerator(object):
         self.clear()
 
     def clear(self):
-        self.namespaces = ["cv"]
-        self.classes = { "Mat" : ClassInfo([ 'class Mat', '', ['/Simple'], [] ], self.namespaces) }
+        self.namespaces = set(["cv"])
+        self.classes = { "Mat" : ClassInfo([ 'class Mat', '', [], [] ], self.namespaces) }
         self.module = ""
         self.Module = ""
         self.ported_func_list = []
@@ -393,15 +408,10 @@ class JavaWrapperGenerator(object):
         if name in type_dict and not classinfo.base:
             logging.warning('duplicated: %s', classinfo)
             return
-        if self.isSmartClass(classinfo):
-            jni_name = "*((*(Ptr<"+classinfo.fullName(isCPP=True)+">*)%(n)s_nativeObj).get())"
-        else:
-            jni_name = "(*("+classinfo.fullName(isCPP=True)+"*)%(n)s_nativeObj)"
         type_dict.setdefault(name, {}).update(
             { "j_type" : classinfo.jname,
               "jn_type" : "long", "jn_args" : (("__int64", ".nativeObj"),),
-              "jni_name" : jni_name,
-              "jni_type" : "jlong",
+              "jni_name" : "(*("+classinfo.fullName(isCPP=True)+"*)%(n)s_nativeObj)", "jni_type" : "jlong",
               "suffix" : "J",
               "j_import" : "org.opencv.%s.%s" % (self.module, classinfo.jname)
             }
@@ -409,8 +419,7 @@ class JavaWrapperGenerator(object):
         type_dict.setdefault(name+'*', {}).update(
             { "j_type" : classinfo.jname,
               "jn_type" : "long", "jn_args" : (("__int64", ".nativeObj"),),
-              "jni_name" : "&("+jni_name+")",
-              "jni_type" : "jlong",
+              "jni_name" : "("+classinfo.fullName(isCPP=True)+"*)%(n)s_nativeObj", "jni_type" : "jlong",
               "suffix" : "J",
               "j_import" : "org.opencv.%s.%s" % (self.module, classinfo.jname)
             }
@@ -521,9 +530,9 @@ class JavaWrapperGenerator(object):
             includes.append('#include "' + hdr + '"')
         for hdr in srcfiles:
             decls = parser.parse(hdr)
-            self.namespaces = sorted(parser.namespaces)
+            self.namespaces = parser.namespaces
             logging.info("\n\n===== Header: %s =====", hdr)
-            logging.info("Namespaces: %s", sorted(parser.namespaces))
+            logging.info("Namespaces: %s", parser.namespaces)
             if decls:
                 includes.append('#include "' + hdr + '"')
             else:
@@ -545,7 +554,7 @@ class JavaWrapperGenerator(object):
         moduleCppCode = StringIO()
         package_path = os.path.join(output_java_path, module)
         mkdir_p(package_path)
-        for ci in sorted(self.classes.values(), key=lambda x: x.symbol_id):
+        for ci in self.classes.values():
             if ci.name == "Mat":
                 continue
             ci.initCodeStreams(self.Module)
@@ -569,7 +578,7 @@ class JavaWrapperGenerator(object):
         report.write("\n".join(self.ported_func_list))
         report.write("\n\nSKIPPED FUNCs LIST (%i of %i):\n\n" % (len(self.skipped_func_list), total_count))
         report.write("".join(self.skipped_func_list))
-        for i in sorted(self.def_args_hist.keys()):
+        for i in self.def_args_hist.keys():
             report.write("\n%i def args - %i funcs" % (i, self.def_args_hist[i]))
         return report.getvalue()
 
@@ -778,7 +787,6 @@ class JavaWrapperGenerator(object):
                             inCode = True
                         if "</code>" in line:
                             inCode = False
-                        line = line.replace('@result ', '@return ')  # @result is valid in Doxygen, but invalid in Javadoc
                         if "@return " in line:
                             returnTag = True
 
@@ -894,17 +902,11 @@ class JavaWrapperGenerator(object):
             elif "v_type" in type_dict[fi.ctype]: # c-tor
                 if type_dict[fi.ctype]["v_type"] in ("Mat", "vector_Mat"):
                     ret = "return (jlong) _retval_;"
-            elif fi.ctype == "String":
+            elif fi.ctype in ['String', 'string']:
                 ret = "return env->NewStringUTF(_retval_.c_str());"
                 default = 'return env->NewStringUTF("");'
             elif self.isWrapped(fi.ctype): # wrapped class:
-                ret = None
-                if fi.ctype in self.classes:
-                    ret_ci = self.classes[fi.ctype]
-                    if self.isSmartClass(ret_ci):
-                        ret = "return (jlong)(new Ptr<%(ctype)s>(new %(ctype)s(_retval_)));" % { 'ctype': self.fullTypeName(fi.ctype) }
-                if ret is None:
-                    ret = "return (jlong) new %s(_retval_);" % self.fullTypeName(fi.ctype)
+                ret = "return (jlong) new %s(_retval_);" % self.fullTypeName(fi.ctype)
             elif fi.ctype.startswith('Ptr_'):
                 c_prologue.append("typedef Ptr<%s> %s;" % (self.fullTypeName(fi.ctype[4:]), fi.ctype))
                 ret = "return (jlong)(new %(ctype)s(_retval_));" % { 'ctype':fi.ctype }
@@ -927,6 +929,8 @@ class JavaWrapperGenerator(object):
                 retval = ""
             elif fi.ctype == "String":
                 retval = "cv::" + self.fullTypeName(fi.ctype) + " _retval_ = "
+            elif fi.ctype == "string":
+                retval = "std::string _retval_ = "
             elif "v_type" in type_dict[fi.ctype]: # vector is returned
                 retval = type_dict[fi.ctype]['jni_var'] % {"n" : '_ret_val_vector_'} + " = "
                 if type_dict[fi.ctype]["v_type"] in ("Mat", "vector_Mat"):
@@ -1043,11 +1047,10 @@ JNIEXPORT $rtype JNICALL Java_org_opencv_${module}_${clazz}_$fname
         if ci.consts:
             enumTypes = set(map(lambda c: c.enumType, ci.consts))
             grouped_consts = {enumType: [c for c in ci.consts if c.enumType == enumType] for enumType in enumTypes}
-            for typeName in sorted(grouped_consts.keys(), key=lambda x: str(x) if x is not None else ""):
-                consts = grouped_consts[typeName]
+            for typeName, consts in grouped_consts.items():
                 logging.info("%s", consts)
                 if typeName:
-                    typeNameShort = typeName.rsplit(".", 1)[-1]
+                    typeName = typeName.rsplit(".", 1)[-1]
 ###################### Utilize Java enums ######################
 #                    ci.j_code.write("""
 #    public enum {1} {{
@@ -1057,19 +1060,19 @@ JNIEXPORT $rtype JNICALL Java_org_opencv_${module}_${clazz}_$fname
 #        {1}(int id) {{ this.id = id; }}
 #        {1}({1} _this) {{ this.id = _this.id; }}
 #        public int getValue() {{ return id; }}
-#    }}\n\n""".format((",\n"+" "*8).join(["%s(%s)" % (c.name, const_value(c.value)) for c in consts]), typeName)
+#    }}\n\n""".format((",\n"+" "*8).join(["%s(%s)" % (c.name, c.value) for c in consts]), typeName)
 #                    )
 ################################################################
                     ci.j_code.write("""
-    // C++: enum {1} ({2})
+    // C++: enum {1}
     public static final int
-            {0};\n\n""".format((",\n"+" "*12).join(["%s = %s" % (c.name, const_value(c.value)) for c in consts]), typeNameShort, typeName)
+            {0};\n\n""".format((",\n"+" "*12).join(["%s = %s" % (c.name, c.value) for c in consts]), typeName)
                     )
                 else:
                     ci.j_code.write("""
     // C++: enum <unnamed>
     public static final int
-            {0};\n\n""".format((",\n"+" "*12).join(["%s = %s" % (c.name, const_value(c.value)) for c in consts]))
+            {0};\n\n""".format((",\n"+" "*12).join(["%s = %s" % (c.name, c.value) for c in consts]))
                     )
         # methods
         for fi in ci.getAllMethods():
@@ -1088,12 +1091,10 @@ JNIEXPORT $rtype JNICALL Java_org_opencv_${module}_${clazz}_$fname
 
         # manual ports
         if ci.name in ManualFuncs:
-            for func in sorted(ManualFuncs[ci.name].keys()):
-                logging.info("manual function: %s", func)
-                fn = ManualFuncs[ci.name][func]
-                ci.j_code.write("\n".join(fn["j_code"]))
-                ci.jn_code.write("\n".join(fn["jn_code"]))
-                ci.cpp_code.write("\n".join(fn["cpp_code"]))
+            for func in ManualFuncs[ci.name].keys():
+                ci.j_code.write ( "\n".join(ManualFuncs[ci.name][func]["j_code"]) )
+                ci.jn_code.write( "\n".join(ManualFuncs[ci.name][func]["jn_code"]) )
+                ci.cpp_code.write( "\n".join(ManualFuncs[ci.name][func]["cpp_code"]) )
 
         if ci.name != self.Module or ci.base:
             # finalize()
@@ -1143,7 +1144,17 @@ JNIEXPORT void JNICALL Java_org_opencv_%(module)s_%(j_cls)s_delete
         if ci.smart != None:
             return ci.smart
 
-        ci.smart = True  # smart class is not properly handled in case of base/derived classes
+        # if parents are smart (we hope) then children are!
+        # if not we believe the class is smart if it has "create" method
+        ci.smart = False
+        if ci.base or ci.name == 'Algorithm':
+            ci.smart = True
+        else:
+            for fi in ci.methods:
+                if fi.name == "create":
+                    ci.smart = True
+                    break
+
         return ci.smart
 
     def smartWrap(self, ci, fullname):
@@ -1311,7 +1322,7 @@ if __name__ == "__main__":
     # initialize logger
     logging.basicConfig(filename='gen_java.log', format=None, filemode='w', level=logging.INFO)
     handler = logging.StreamHandler()
-    handler.setLevel(os.environ.get('LOG_LEVEL', logging.WARNING))
+    handler.setLevel(logging.WARNING)
     logging.getLogger().addHandler(handler)
 
     # parse command line parameters
